#include <stdio.h> 
#include <netdb.h>
#include <libgen.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <globus_gram_client.h>
#include <globus_nexus.h>

#include <globus_toolkit.h>

static void gs_remote_handler(globus_nexus_endpoint_t *endpoint,
			      globus_nexus_buffer_t *buffer,
			      globus_bool_t called_from_non_threaded_handler);
#define GU_REMOTE_HANDLER_ID 0
#define GS_HANDLER_TYPE GLOBUS_NEXUS_HANDLER_TYPE_NON_THREADED

static globus_nexus_handler_t gs_remote_handler_table[] =
{
  {GS_HANDLER_TYPE,
   gs_remote_handler},
};

#define GU_REMOTE_HANDLER_TABLE_SIZE 1

/* this is the function called when someone makes an attach request */
/* on the port I specifiy - should I put some sort of authentication */
/* stuff here, like "who was buried in grant's tomb?" or something? */
static int gs_attach_requested(void *user_arg, char *url, globus_nexus_startpoint_t *sp);

/* this is the func used to report GRAM_JOB_MANAGER information */
/* currently it doesn't do anything much, but we can use this for */
/* reporting all of the relevant process status information back */
/* the CAT */
static void gs_gram_callback_func(void *user_callback_arg,
				  char *job_contact,
				  int state,
				  int errorcode);


/* startpoint list */
sp_list *spListHead=NULL;
globus_mutex_t *spListMutex;

void spListAdd(sp_list *newLink);
void mark_for_deletion(sp_list *link);

/* variables for GRAM stuff */
static char gs_contact[1024];
static char gs_contact_hostname[1024];
static char gs_process_hostname[1024];
static char gs_spec[1024];
static char gs_remote_machine[256];
static int gs_job_state_mask=31;
static char *gs_job_contact;
static char *gs_callback_contact;

/* variables for NEXUS stuff */
static globus_nexus_endpoint_t *gs_localEP;
static char gs_attach_host[1024];
static int gs_attach_port;

/* global process counter varible */
static int procCounter=0;

/* gs_init : call this to get everything organized */
/* returns GRAM_INIT_FAILED or ALLOW_ATTACH_FAILED for failures, */
/* GLOBUS_SUCCESS for success */
int gs_init(void)
{
  int rc;
  unsigned short port=0;
  char *host;

  if((rc = globus_module_activate(GLOBUS_NEXUS_MODULE)) != GLOBUS_SUCCESS && rc!=GLOBUS_NEXUS_ERROR_ALREADY_INITIALIZED) return GS_NEXUS_INIT_FAILED;
  if((rc = globus_module_activate(GLOBUS_GRAM_CLIENT_MODULE)) != GLOBUS_SUCCESS) return GS_GRAM_CLIENT_INIT_FAILED;

  /* set up our local nexus guys */
  gs_localEP=(globus_nexus_endpoint_t *) malloc(sizeof(globus_nexus_endpoint_t));
  /* now for the nexus callbacks, slightly more work */
  /* which I've squirled away in some utility funcs in the */
  /* globus_toolkit library */
  globus_endpoint_setup(gs_localEP, gs_remote_handler_table,
			GU_REMOTE_HANDLER_TABLE_SIZE);
			

  /* Here's where we fill in our attachment information which we */
  /* will be passing to the created process through its argv */
  /* variables. */
  rc = globus_nexus_allow_attach(&port, &host,
				 &gs_attach_requested,
				 (void *) gs_localEP);
  
  if(rc!=GLOBUS_SUCCESS) return GS_ALLOW_ATTACH_FAILED;

  strcpy(gs_attach_host, host);
  gs_attach_port=port;
  
  spListMutex = (globus_mutex_t *) globus_malloc (sizeof (globus_mutex_t));
  globus_mutex_init (spListMutex, NULL);
  
  return rc;
} 

/* gs_connect : collects information on the machine we're going to try */
/* to spork our process on - doesn't actually do anything but fill */
/* in some information for use later */
int gs_connect(char *hostname)
{
  int rc;
  struct hostent *myhost;
  
  myhost = gethostbyname( hostname );
  sprintf(gs_contact_hostname, "%s", myhost->h_name);
  getContactString(gs_contact_hostname, gs_contact);
  printf("%s\n", gs_contact);
  fflush(NULL);
  return GLOBUS_SUCCESS;
}

/* gs_create_process : Here's the big one - uses GRAM to spork */
/* a new process on the remote machine, then sits and waits */
/* on a globus_monitor until it contacts us back on the */
/* port we give it so we can trade startpoints. */
globus_nexus_startpoint_t *gs_create_process(char *comp, char *machine, char *path)
{
  int rc;
  char *directory, *executable, tmp[1024], tmp2[1024];
  sp_list *newLink;
  struct hostent *myhost;

  strcpy(tmp, path);
  strcpy(tmp2, path);
  directory=dirname(tmp);
  executable=basename(tmp2);

  myhost = gethostbyname( machine );
  sprintf(gs_process_hostname, "%s", myhost->h_name);
  newLink=(sp_list *)globus_malloc(sizeof(sp_list));
  newLink->remoteSP=(globus_nexus_startpoint_t *) globus_malloc(sizeof(globus_nexus_startpoint_t));
  newLink->Monitor=(globus_monitor_t *)globus_malloc(sizeof(globus_monitor_t));
  globus_monitor_init(newLink->Monitor);
  newLink->procNum=procCounter++;
  newLink->deleteFlag=0;
  spListAdd(newLink);
  
  /* set up the gram callback */
  globus_gram_client_callback_allow(gs_gram_callback_func,
				    (void *) newLink,
				    &newLink->callback_contact);
  

  if(!strcmp(gs_process_hostname, gs_contact_hostname)) {
  	sprintf(gs_spec, "&(directory=\"%s\")(executable=\"%s\")(stdout=$(HOME)/ # \"%s.out\")(stderr=$(HOME)/ # \"%s.err\")(count=1)(arguments=%d %s %d)", directory, executable, executable, executable, gs_attach_port, gs_attach_host, newLink->procNum);
  }
  else {
  	sprintf(gs_spec, "&(class = g)(requirements = \"( Machine == %s )\")(directory=\"%s\")(executable=\"%s\")(stdout=$(HOME)/ # \"%s.out\")(stderr=$(HOME)/ # \"%s.err\")(count=1)(arguments=%d %s %d)", machine, directory, executable, executable, executable, gs_attach_port, gs_attach_host, newLink->procNum);
  }

  /* make the request to the gatekeeper for the process */
  rc = globus_gram_client_job_request(gs_contact,
				      gs_spec,
				      gs_job_state_mask,
				      newLink->callback_contact,
				      &(newLink->job_contact));
  
  if(rc != GLOBUS_SUCCESS) {
	globus_libc_printf("createProcess: ERROR: %d\n", rc);
	gs_disconnect(newLink, GS_MODE_NORMAL);
    return NULL;
  }
  /* sit and wait for it to contact us */
  globus_monitor_wait(newLink->Monitor);

  /* do some housekeeping, deleting any stray nodes in the sp_list */
  gs_disconnect((sp_list *)NULL, GS_MODE_CLEANUP);

  /* Once we are here we're attached */
  /* and we got a startpoint - let's return it */
  return newLink->remoteSP;
}

void spListAdd(sp_list *newLink)
{
  globus_mutex_lock (spListMutex);

  newLink->next = spListHead;
  spListHead=newLink;

  globus_mutex_unlock (spListMutex);
}  

/* attach_requested : this is the handler function for the attach */
/* request which will come in from created processes. */
static int gs_attach_requested(void *user_arg, char *url, globus_nexus_startpoint_t *sp)
{
  /* I think I can use remoteEP here - I'll check */
  globus_nexus_endpoint_t *ep = (globus_nexus_endpoint_t *) user_arg;

  globus_nexus_startpoint_bind(sp, ep);
  return(0);
  
}


/* gs_disconnect : handles getting rid of the startpoint associated */
/* with the remote process, which we either killed, or it managed */
/* to die all on its own... */
int gs_disconnect(sp_list *link, int mode)
{
  sp_list *here,  /* the current element */
    *prev,        /* the current element's previous element in the linked list */
    *toBeDeleted; /* the element to be deleted */
  int rc = 0;     /* currently unused */

  globus_mutex_lock (spListMutex);
  
  for(here = spListHead, prev = here; here;) {
	 
    if((mode == GS_MODE_CLEANUP && here->deleteFlag) || 
	  (mode == GS_MODE_NORMAL && here == link)) {
	 
	 toBeDeleted = here;
	 
      globus_monitor_destroy(here->Monitor);

	 if (mode == GS_MODE_CLEANUP) {
	   globus_nexus_startpoint_destroy(here->remoteSP);
	   globus_free(here->callback_contact);
	   globus_free(here->job_contact);
	 }
	 globus_free (here->remoteSP);
	 
	 if (spListHead == here)  /* I am deleting the entry directly pointed to be 
						    the spListHead */
	   spListHead = spListHead->next;
	 if (prev != here)
	   prev->next = here->next;
	 else
	   prev = prev->next;
	 here = here->next;

	 globus_free(toBeDeleted);
    }
    else {
	 if (prev != here)
	   prev = prev->next;
	 here = here->next;
    }
  }

  globus_mutex_unlock (spListMutex);

  return rc;
}


void gs_cleanup(void)
{
  globus_module_deactivate_all();
}


/* gram_callback_func : where our GRAM notifications are received */
/* and processed - righ tnow we dont do much except make sure */
/* we know when stuff is dead. */
static void gs_gram_callback_func(void *user_callback_arg,
			       char *job_contact,
			       int state,
			       int errorcode)
{
  sp_list *link=(sp_list *)user_callback_arg;
  
  globus_mutex_lock (spListMutex);
  
  switch(state)
  {
  case GLOBUS_GRAM_CLIENT_JOB_STATE_PENDING:  break;
  case GLOBUS_GRAM_CLIENT_JOB_STATE_ACTIVE:     
    break;
  case GLOBUS_GRAM_CLIENT_JOB_STATE_FAILED:
    mark_for_deletion(link);
    break;
  case GLOBUS_GRAM_CLIENT_JOB_STATE_DONE:
    mark_for_deletion(link);
    break;
  }

  globus_mutex_unlock (spListMutex);
}

void mark_for_deletion(sp_list *link)
{
	link->deleteFlag=1;
}

/*
 * cat_remote_handler()
 */
static void gs_remote_handler(globus_nexus_endpoint_t *endpoint,
			       globus_nexus_buffer_t *buffer,
			       globus_bool_t called_from_non_threaded_handler)
{
  globus_nexus_buffer_t reply_buffer;
  int procNum;
  sp_list *spTmp;
  
  /* get the ID of the sending process */
  globus_nexus_get_int(buffer, &procNum, 1);

  globus_mutex_lock (spListMutex);

  for(spTmp=spListHead; spTmp && spTmp->procNum!=procNum; spTmp=spTmp->next);

  if(!spTmp) {
    globus_nexus_printf("process ID not found :  %d!\n", procNum);
    globus_nexus_printf("(you may have deleted it yourself)\n");

    globus_mutex_unlock (spListMutex);

    return;
  }
  
  /* Get the remote global pointer */
  globus_nexus_get_startpoint(buffer, spTmp->remoteSP, 1);

  /* free the buffer up */
  globus_nexus_buffer_destroy(buffer);
  
  /* free ourselves for further action */
  globus_monitor_release(spTmp->Monitor, GLOBUS_MONITOR_SUCCESS);

  globus_mutex_unlock (spListMutex);

  return;
}
