//-----------------------------------------------------------------------------
// bhp_Comm.h -- Category of objects used for communication.  For example,
// Remote Method Invocation (RMI) and global pointer get/put.
//-----------------------------------------------------------------------------

#ifndef __HBP_COMM_H
#define __HBP_COMM_H

#include <HPCxx_Comm.h>
#include <hpcxx_types.h>
#include <HPCxx_Buffer.h>
#include <HPCxx_Sync.h>
#include <iostream.h>
#include <hbp.h>


class hbp_CmdHdr {
  short senderId;
  short threadId;
  short msgType;
  short msgId;
  short senderIndex;
  short unused;
  char* data;		// pointer to the data/message portion of the packet
};


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
class hbp_Comm : public HPCxx_Comm {

 public:

  hbp_Comm(HPCxx_Mutex* mutex, hbp_state* hbp, unsigned int dst_pg,
	    unsigned int dst_off, unsigned int dst_port,
	    PageBuffer* sendBuf, PageBuffer* ackBuf);
  virtual ~hbp_Comm(void);

  virtual void send(void);
  virtual void send(HPCxx_Buffer& buffer);
  virtual void send(void* buf, size_t size);
  virtual void recv(void* buf, size_t size);

  bool inUse(void)		{ return inUse; }
  void setInUse(bool isInUse)	{ inUse = isInUse; }
  void setSenderIndex(int index);

 private:

  hbp_state* hbp;	     // hippi bypass state object
  unsigned int dst_pg;	     // receiver page number
  unsigned int dst_off;	     // receiver page offset (in words, scache aligned)
  unsigned int dst_port;     // port to use at destination
  PageBuffer* sendBuf;	     // fixed size buffer for command/control msgs
  PageBuffer* ackBuf;	     // fixed size buffer for ack of cmd msgs
  bool inUse;		     // comm object is in use, must wait for ack

  static int sender(void* buf);
  static int senderIndex(void* buf);

};


//
// inline functions
//

inline int hbp_Comm::sender(void* buf)
  {
    return (int) ((hbp_CmdHdr*) buf)->senderID;
  }

inline int hbp_Comm::senderIndex(void* buf)
  {
    return (int) ((hbp_CmdHdr*) buf)->senderIndex;
  }


#endif // __HBP_COMM_H
