#ifndef __HPCXX_COMMCOORD_H_
#define __HPCXX_COMMCOORD_H_

#include "HPCxx_Comm.h"
#include "HPCxx_Mutex.h"
#include <vector.h>
#include <deque.h>
#include <list.h>

class hbp_CommCoord : public HPCxx_CommCoord {

 public:

  hbp_CommCoord(void);
  ~hbp_CommCoord(void);

  //#---- communicator --------------------------------------------------------
  // Returns a communications object.
  // <ul>
  // <li><b>dest</b> -- the destination of the message
  // <li><b>msgType</b> -- the message type to send
  // <li><b>unique</b> -- use a unique message id if true
  // </ul>
  //#--------------------------------------------------------------------------
  HPCxx_Comm* communicator(const HPCxx_ContextID& dest, int msgType,
			   bool unique = false);


  //#---- handleMessage -------------------------------------------------------
  // Handles an incoming message.
  // <ul>
  // <li><b>status</b> -- the status of the message (tag, sender, size)
  //#--------------------------------------------------------------------------
  void handleMessage(HPCxx_Comm& comm);

  //#---- release -------------------------------------------------------------
  // Releases what had been an "in use" communications object.
  // <ul>
  // <li><b>comm</b> -- the communication object to release
  // </ul>
  //#--------------------------------------------------------------------------
  void release(HPCxx_Comm* comm);

  //#---- releaseToPosted -----------------------------------------------------
  // Releases a communications object back to the coordinator to be placed
  // in the posted queue.
  // <ul>
  // <li><b>comm</b> -- the communication object to release
  // </ul>
  //#--------------------------------------------------------------------------
  void releaseToPosted(HPCxx_Comm* comm);

  //#---- releaseToPreposted --------------------------------------------------
  // Releases a communications object back to the coordinator to be placed
  // in the preposted queue (those objects that have preposted a receive).
  // <ul>
  // <li><b>comm</b> -- the communication object to release
  // </ul>
  //#--------------------------------------------------------------------------
  void releaseToPreposted(HPCxx_Comm* comm);

  //#---- tag -----------------------------------------------------------------
  // Returns a tag based on message type, message id, and kernel-level thread #
  // <ul>
  // <li><b>msgType</b> -- the message type to use in constructing the tag
  // <li><b>msgID</b> -- the message id to use in constructing the tag
  // </ul>
  //#--------------------------------------------------------------------------
  static int tag(int msgType, int msgID);

  //#---- commMutex -----------------------------------------------------------
  // Returns the communication mutex to be used to protect communication
  // critical sections.
  //#--------------------------------------------------------------------------
  static HPCxx_Mutex* commMutex(void);

  //#---- setCommMutex --------------------------------------------------------
  // Sets the communication mutex to be used to protect communication critical
  // sections.
  //#--------------------------------------------------------------------------
  static void setCommMutex(HPCxx_Mutex* mutex);

  //#---- poll ----------------------------------------------------------------
  // Polls communication queues for events.
  //#--------------------------------------------------------------------------
  void poll(void);

 private:

  int context;			// my context number
  int nContexts;		// # of cooperating contexts

  int* nComms;			// # of send object (for each context)
  int* nextSend;		// index of next send object (for each context)

  vector<hbp_SendComm*>* sendQ;	// array of containers for context send objs
  vector<hbp_RecvComm*>* recvQ;	// array of containers for context recv objs

};

#endif // __HPCXX_COMMCOORD_H_
