// HPC++ Run Time Sytem Library
// Extreme! Computing Lab - Computer Science Department
// Copyright (c) 1997 Indiana University
// Authors: Pete Beckman, Dennis Gannon, Todd Green, and Liz Johnson
//
// $Id: HPCxx_ThreadImpl.h,v 1.3 1999/01/29 23:47:20 rasmussn Exp $

#ifndef __HPCXX_THREADIMPL_H_
#define __HPCXX_THREADIMPL_H_

#include <Smarts.h>
#include <Thread.h>
#include <hpcxx_types.h>
#include <HPCxx_Runnable.h>

class HPCxx_Thread;

class HPCxx_TulipThread : public Smarts::Thread {
private:
  HPCxx_Runnable* runnable_;
  HPCxx_Thread* hpcxx_thread_;

public:
  HPCxx_TulipThread (void) : Smarts::Thread("HPC++"),
    runnable_((HPCxx_Runnable *)0), hpcxx_thread_((HPCxx_Thread *)0) { }
  
  HPCxx_TulipThread (HPCxx_Thread* hpcxx_thread) : Smarts::Thread("HPC++"),
    runnable_((HPCxx_Runnable *)0), hpcxx_thread_(hpcxx_thread) { }
  
  HPCxx_TulipThread (HPCxx_Runnable* runnable, HPCxx_Thread* hpcxx_thread) :
    Smarts::Thread("HPC++"), runnable_(runnable), hpcxx_thread_(hpcxx_thread)
    { }
  
  void run (void);

  friend class HPCxx_ThreadImpl;
};


class HPCxx_ThreadImpl {
private:
  HPCxx_TulipThread* thread_;

  static void* start_(void* rhs) { return ((void *) 0); }

public:
  HPCxx_ThreadImpl(void);
  HPCxx_ThreadImpl(HPCxx_Thread* hpcxx_thread);
  HPCxx_ThreadImpl(const HPCxx_ThreadImpl& thread);
  HPCxx_ThreadImpl(HPCxx_Runnable *runnable, HPCxx_Thread* hpcxx_thread);
  HPCxx_ThreadImpl& operator=(const HPCxx_ThreadImpl& thread);
  
  virtual ~HPCxx_ThreadImpl(void);
  
  virtual void run(void) { thread_->run(); }
  
  // Starts execution of this thread and calls the run() method.
  inline void start(HPCxx_Thread& thread)
  {
//    if (thread_->isAlive())
//      return;

    thread_->start();
  }
  
  // Ceases execution of this thread.  Status is not currently used.
  inline void stop(void *status) { thread_->stop(status); }

  // Causes the currently executing thread object to temporarily pause and
  // allow other threads to execute.
  inline static void yield(void) { Smarts::Context::thread()->yield(); }

  // Causes the currently executing thread to sleep (cease executing) for the
  // specified number of milliseconds plus the specified number of nanoseconds.
  // The sleep period will be rounded up to the resolution of the system timer.
  inline static void sleep(long milliseconds, long nanoseconds = 0)
  {
    Smarts::Context::thread()->sleep(milliseconds, nanoseconds);
  }

  // Suspends execution of this thread until resume is called.
  inline void suspend(void) { thread_->suspend(); }
  
  // Resume a suspended thread.
  inline void resume(void) { thread_->resume(); }
  
  // Check to see if thread has been started but has not yet died.
  inline bool isAlive(void) { return thread_->isAlive(); }
  
  // Returns a reference to the currently executing thread object.
  inline static HPCxx_Thread *currentThread(void)
  {
    return (HPCxx_Thread *) Smarts::Context::thread();
  }
  // Returns the numeric ID of the currently executing thread object.
  inline static int getThreadID(void)
  {
    return Smarts::Context::thread()->getThreadID();
  }

  // Join waits for the completion of this thread.  By definition join returns
  // as soon as this thread is considered not alive.
  inline void join(int milliseconds, int nanoseconds = 0)
  {
    Smarts::Thread::join(*thread_);
  }
  
  // Returns this thread`s priority, where
  // HPCXX_MIN_THREAD_PRIORITY <= getPriority() <= HPCXX_MAX_THREAD_PRIORITY
  inline int getPriority(void) { return thread_->priority(); }
  
  // Changes the priority of this thread.  Priority is clamped to
  // (HPCXX_MIN_THREAD_PRIORITY, HPCXX_MAX_THREAD_PRIORITY)
  inline int setPriority(int priority)
  {
    thread_->priority(priority);
    return thread_->priority();
  }
}; 

#endif 


