#include <sys/utsname.h>
#include "templ.h"
#include <stdlib.h>
#include <strings.h>
#include <hpcxx_rts.h>
#include "HPCxx_String.h"
#include "MyArray.h"
#include "globus_toolkit.h"
#include <sys/time.h>
//simple event timer macro
#define ElapseTime(Start, Stop) \
    (  (  ( ((double)(Stop.tv_sec)) - ((double)(Start.tv_sec)) ) \
        + (((double)(Stop.tv_usec))/1000000.0) ) \
     - (((double)(Start.tv_usec))/1000000.0) )

// Make sure that the command line arguments are correctly passed on by the
// user.
void parseArguments(int argc) {  
  if (argc != 4) {
    cout << "Please check your command line arguments \n";
    cout << "GramClient <GATEKEEPER_HOST> <EXECUTABLE_HOST> <EXECUTABLE_PATH>\n";
    cout << "... Exiting ...\n";
    exit(EXIT_FAILURE);
  }
}

// need a context id to hold reference to remote context.
 HPCxx_ContextID *remoteCID = NULL; 

// ids used in server for its externally called functions
#define PASS_ARRAY_ID        179
#define PASS_DATA_ID         180
#define SAY_HELLO_ID         181
#define KILL_ME_ID           182
// which correspond to these functions (which are included as
// dummy stubs here... a prototype is not sufficient for the templates.

int          passArray(MyArray a){ /* dummy version */ return 0; }
float        passData(float x, int y, int z){ /* dummy version */ return 0; }
HPCxx_String sayHello(HPCxx_String){ /* dummy version*/  return NULL; }
int          killMe(){ /* dummy version */ return 0; }

void runClient(char **argv) {
  char *gateKeeperMachine = strdup(argv[1]);
  char *executableMachine = strdup(argv[2]);
  char *executablePath = strdup(argv[3]);

 // register all the required functions
  hpcxx_register(passArray, PASS_ARRAY_ID);
  hpcxx_register(passData, PASS_DATA_ID);
  hpcxx_register(sayHello, SAY_HELLO_ID);
  hpcxx_register(killMe, KILL_ME_ID);

  // Initialize GlobusToolkit. This must be done only once in the context.
  int errorval = gs_init();
  if(errorval) {
    cout << "Client:: gs_init() failed with return value " << errorval << endl;
    cout << ".. Exiting ...  \n";
    exit(EXIT_FAILURE);
  } 

  // make contact with the gatekeeper
  hpcxx_connectContext(gateKeeperMachine); 

  // Create a context on the target machine, and get its contextID
  // back. 
  remoteCID = hpcxx_createContext(executableMachine, executablePath);
  if (!remoteCID)  {
    cout << "NULL remoteCID  received by the Gramtest client\n";
    cout << "... Exiting ... \n";
    exit(EXIT_FAILURE);
  }

 // ----------------------------------------------------------
 // now leets call the say hello remote function.

  HPCxx_String returnedVal;
  HPCxx_String passedVal("client side");

  hpcxx_invoke(*remoteCID, returnedVal, SAY_HELLO_ID, passedVal);
  cout << "invoke for sayhello returned with value:" << returnedVal << endl;

  // -----------------------------------------------------------
  // now make 1000 small remote function calls and time them
  int i;
  timeval start_time, stop_time;
  float elap_time;
  float w, q = 0.032;
  int j = 2000;

  cout << "doing small message round-trip cost timing" << endl;
  gettimeofday(&start_time, NULL);
  for(i = 0; i < 1000; i++){
    hpcxx_invoke(*remoteCID, w, PASS_DATA_ID, q, j, i);
    //cout << "w = " << w << endl;
   } 
  gettimeofday(&stop_time, NULL);

  elap_time = ElapseTime(start_time, stop_time);  
  cout << "time per call = " << elap_time << " ms" << endl;

  // -----------------------------------------------------------
  // now lets create a big array of doubles and pass and time
  // them.
  const int ar_size = 1000000;
  float *ar = new float[ar_size];
  float *ar2 = new float[1];
  if((ar==NULL) || (ar2 == NULL)){
      cout << "bad array alloc" << endl;
      cout.flush();
      exit(0);
     }
  MyArray db_ar(ar, ar_size);
  for(i = 0; i < ar_size; i++) db_ar[i] = i;
  MyArray db_ar2(ar2, 1);
  db_ar2[0] = 1.0;
  int v = 0;

  gettimeofday(&start_time, NULL);
  hpcxx_invoke(*remoteCID, v, PASS_ARRAY_ID, db_ar2);
  gettimeofday(&stop_time, NULL);
  float elap_time2 = ElapseTime(start_time, stop_time);  
  gettimeofday(&start_time, NULL);
  cout << "done with first\n";
  cout.flush();
  int v2;
  gettimeofday(&start_time, NULL);
  hpcxx_invoke(*remoteCID, v2, PASS_ARRAY_ID, db_ar);
  gettimeofday(&stop_time, NULL);
  elap_time = ElapseTime(start_time, stop_time);  
  float bw = (ar_size - 1.0)/(elap_time-elap_time2);
  float lat = elap_time - ar_size/bw;
  float lat2 = elap_time2 - 1.0/bw;
  bw = 8.0*sizeof(float)*bw/1.0e6;
  cout << "Bandwidth = " << bw << " Mbps.  latency = " << lat*1000.0 << 
   " ms" << " lat2 =" << lat2*1000.0 << endl;
  cout.flush(); 

  free(gateKeeperMachine);
  free(executableMachine);
  free(executablePath);
}

void shutdown(){
  // shut down server
  int x;
  if(remoteCID){
      hpcxx_invoke(*remoteCID, x, KILL_ME_ID);
      cout << "return value from killMe = " << x << endl;

      hpcxx_destroyContextID(remoteCID);

      cout << "Server instantiated and killed !!\n";
      cout << "test Complete\n";
     }
}

  
static HPCxx_Group *group;

int main(int argc, char **argv) {
  hpcxx_init(argc, argv, group);
  parseArguments(argc);
  runClient(argv);
  shutdown();
  hpcxx_exit();
}
 
