//##########################################################################
// HPC++Lib: An object-oriented class library for high performance 
//           parallel programming.
//
// Copyright (c) 1997
//	Indiana University
//	Los Alamos National Laboratory
// 
// http://www.acl.lanl.gov/Tulip
// http://www.extreme.indiana.edu/HPC++
// 
// Pete Beckman, Dennis Gannon, Todd Green, Liz Johnson
//
// HPC++_VER: $Id: HPCxx_Group.C,v 1.1 1999/04/17 01:00:21 kate Exp $
//##########################################################################

// $Id: HPCxx_Group.C,v 1.1 1999/04/17 01:00:21 kate Exp $

#include <string.h>

#include <HPCxx_Group.h>

HPCxx_Group::HPCxx_Group(hpcxx_id_t groupID, const char *name)
  : contextIDs_(NULL), numContextIDs_(0), numThreads_(0)
{
  id_ = (groupID == HPCXX_GEN_LOCAL_GROUPID) ?
	hpcxx_getLocalID(HPCXX_GROUP_IDSPACE) : groupID;
  strcpy(name_, name);
}

// Create a group for the context id and the passed in id
HPCxx_Group::HPCxx_Group(HPCxx_ContextID *&id,
			hpcxx_id_t groupID,
			const char *name)
  : contextIDs_(&id), numContextIDs_(1), numThreads_(0)
{
  id_ = (groupID == HPCXX_GEN_LOCAL_GROUPID) ?
	hpcxx_getLocalID(HPCXX_GROUP_IDSPACE) : groupID;
  strcpy(name_, name);
}

// Create a group
HPCxx_Group::HPCxx_Group(HPCxx_ContextID **id,
			int count,
			hpcxx_id_t groupID,
			const char *name)
  : contextIDs_(id), numContextIDs_(count), numThreads_(0)
{
  id_ = (groupID == HPCXX_GEN_LOCAL_GROUPID) ?
	hpcxx_getLocalID(HPCXX_GROUP_IDSPACE) : groupID;
  strcpy(name_, name);
}


HPCxx_Group::~HPCxx_Group()
{
}
  
hpcxx_id_t &HPCxx_Group::getGroupID()
{
  return(id_);
}

// Get the number of contexts that are participating
// in this group
int HPCxx_Group::getNumContexts()
{
  return(numContextIDs_);
}

// Return an ordered array of context IDs in
// this group.  This array is identical for every member
// of the group.
// Caller must 'delete ' the memory for the GlobalPtrs 
// caveat - getNumContexts may not equal the number of
// contexts in this array
HPCxx_ContextID **HPCxx_Group::getContextIDs()
{
  return(contextIDs_);
}

// Return the context id for zero-based context <n> where
// <n> is less than the current number of contexts
HPCxx_ContextID *HPCxx_Group::getContextID(int context)
{
  return(contextIDs_[context]);
}

// Set the number of threads for this group in *this*
// context.
void HPCxx_Group::setNumThreads(int count)
{
  // FIXME - should we allocated/create the threads here?
  numThreads_ = count;
}

// Return the number of threads for this group in *this* context
int HPCxx_Group::getNumThreads()
{
  return(numThreads_);
}


// Set the name of the current group
void HPCxx_Group::setName(const char *name)
{
  strcpy(name_, name);
}

// Retuns the name of the current group
const char *HPCxx_Group::getName()
{
  return(name_);
}
