//##########################################################################
// HPC++Lib: An object-oriented class library for high performance 
//           parallel programming.
//
// Copyright (c) 1997
//	Indiana University
//	Los Alamos National Laboratory
// 
// http://www.acl.lanl.gov/Tulip
// http://www.extreme.indiana.edu/HPC++
// 
// Pete Beckman, Dennis Gannon, Todd Green, Liz Johnson
//
// HPC++_VER: $Id: hpcxx_register.h,v 1.5 1998/12/18 07:38:32 btemko Exp $
//##########################################################################

#ifndef __HPCXX_REGISTER_H_
#define __HPCXX_REGISTER_H_

//# include files
#include <hpcxx_types.h>
#ifdef TULIP_FORTRAN
#include <HPCxx_Prototype.h>
#endif

// This object provides a mapping between the prototypes and the
// regitrations id that are used to register a function.  A prototype
// (see HPCxx_Prototype.h) is an object containing a function prototype.
#ifdef TULIP_FORTRAN
extern HPCxx_IdProtoMap hpcxx_prototypeMap;
#endif


// register a class
template<class C>
hpcxx_id_t hpcxx_registerClass(C *c); 


// register a global function that takes no arguments
template<class R>
hpcxx_id_t hpcxx_register(R (*f) (void), hpcxx_id_t registerID );

// Register a global function that takes one argument
template<class R, class A1>
hpcxx_id_t hpcxx_register(R (*f) (A1), hpcxx_id_t registerID );

// Register a global function that takes two arguments
template<class R, class A1, class A2>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2), hpcxx_id_t registerID );

// Register a global function that takes three arguments
template<class R, class A1, class A2, class A3>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3), hpcxx_id_t registerID );

// Register a global function that takes four arguments
template<class R, class A1, class A2, class A3, class A4>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4), hpcxx_id_t registerID );

// Register a global function that takes five arguments
template<class R, class A1, class A2, class A3, class A4, class A5>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5), hpcxx_id_t registerID );

// Register a global function that takes six arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6), 
              hpcxx_id_t registerID );

// Register a global function that takes seven arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6, A7), hpcxx_id_t registerID );

// Register a global function that takes eight arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6, A7, A8), hpcxx_id_t registerID );

// Register a member function that takes no arguments
template<class R, class C>
hpcxx_id_t hpcxx_register(R (C::*mf) (void), hpcxx_id_t registerID );

// Register a member function that takes one argument
template<class R, class C, class A1>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1), hpcxx_id_t registerID );

// Register a member function that takes two arguments
template<class R, class C, class A1, class A2>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2), hpcxx_id_t registerID );

// Register a member function that takes three arguments
template<class R, class C, class A1, class A2, class A3>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3), hpcxx_id_t registerID );

// Register a member function that takes four arguments
template<class R, class C, class A1, class A2, class A3, class A4>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4), hpcxx_id_t registerID );

// Register a member function that takes five arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5), hpcxx_id_t registerID );

// Register a member function that takes six arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6), hpcxx_id_t registerID );

// Register a member function that takes seven arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6, A7), hpcxx_id_t registerID );

// Register a member function that takes eight arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6, A7, A8), hpcxx_id_t registerID );

#endif
